package net.knarcraft.blockhunt.config;

import net.knarcraft.blockhunt.BlockHunt;
import net.knarcraft.blockhunt.MemoryStorage;
import net.knarcraft.blockhunt.manager.ConfigManager;

/**
 * A key pointing to a value in config.yml
 */
public enum ConfigKey implements Key {

    CHAT_TAG("[" + BlockHunt.getPluginDescriptionFile().getName() + "] ", "chat.tag"),
    CHAT_NORMAL("&b", "chat.normal"),
    CHAT_WARNING("&c", "chat.warning"),
    CHAT_ERROR("&c", "chat.error"),
    CHAT_ARGUMENT("&e", "chat.arg"),
    CHAT_HEADER("&9", "chat.header"),
    CHAT_HEADER_HIGH("%H_______.[ %A%header%%H ]._______", "chat.headerHigh"),

    COMMAND_ENABLED_INFO(true, "commandEnabled.info"),
    COMMAND_ENABLED_HELP(true, "commandEnabled.help"),
    COMMAND_ENABLED_RELOAD(true, "commandEnabled.reload"),
    COMMAND_ENABLED_JOIN(true, "commandEnabled.join"),
    COMMAND_ENABLED_LEAVE(true, "commandEnabled.leave"),
    COMMAND_ENABLED_LIST(true, "commandEnabled.list"),
    COMMAND_ENABLED_SHOP(true, "commandEnabled.shop"),
    COMMAND_ENABLED_START(true, "commandEnabled.start"),
    COMMAND_ENABLED_WAND(true, "commandEnabled.wand"),
    COMMAND_ENABLED_CREATE(true, "commandEnabled.create"),
    COMMAND_ENABLED_SET(true, "commandEnabled.set"),
    COMMAND_ENABLED_SET_WARP(true, "commandEnabled.setWarp"),
    COMMAND_ENABLED_REMOVE(true, "commandEnabled.remove"),
    COMMAND_ENABLED_TOKENS(true, "commandEnabled.tokens"),

    AUTO_UPDATE_CHECK(true, "autoUpdateCheck"),
    AUTO_DOWNLOAD_UPDATE(false, "autoDownloadUpdate"),

    WAND_ID_NAME("STICK", "wandIdName"),
    WAND_NAME("%A&l" + BlockHunt.getPluginDescriptionFile().getName() + "%N's selection wand", "wandName"),
    WAND_DESCRIPTION(new String[]{"%NUse this item to select an arena for your arena.",
            "%ALeft-Click%N to select point #1.", "%ARight-Click%N to select point #2.",
            "%NUse the create command to define your arena.", "%A/" + BlockHunt.getPluginDescriptionFile().getName() + " <help|h>"},
            "wandDescription"),

    SHOP_TITLE("%H&lBlockHunt %NShop", "shop.title"),
    SHOP_PRICE("%NPrice: %A%amount% %Ntokens.", "shop.price"),

    SHOP_BLOCK_CHOOSER_V_1_ENABLED(true, "shop.blockChooserV1Enabled"),
    SHOP_BLOCK_CHOOSER_V_1_ID_NAME("BOOK", "shop.blockChooserV1IdName"),
    SHOP_BLOCK_CHOOSER_V_1_PRICE(3000, "shop.blockChooserV1Price"),
    SHOP_BLOCK_CHOOSER_V_1_NAME("%H&lBlockHunt Chooser", "shop.blockChooserV1Name"),
    SHOP_BLOCK_CHOOSER_V_1_DESCRIPTION(new String[]{"%NUse this item before the arena starts.",
            "%ARight-Click%N in the lobby and choose", "%Nthe block you want to be!", "&6Unlimited uses."},
            "shop.blockChooserV1Description"),

    SHOP_BLOCK_HUNT_PASS_V_2_ENABLED(true, "shop.blockHuntPassV2Enabled"),
    SHOP_BLOCK_HUNT_PASS_V_2_ID_NAME("NAME_TAG", "shop.blockHuntPassV2IdName"),
    SHOP_BLOCK_HUNT_PASS_V_2_PRICE(150, "shop.blockHuntPassV2Price"),
    SHOP_BLOCK_HUNT_PASS_V_2_NAME("%H&lBlockHunt Pass", "shop.blockHuntPassV2Name"),
    SHOP_BLOCK_HUNT_PASS_V_2_DESCRIPTION(new String[]{"%NUse this item before the arena starts.",
            "%ARight-Click%N in the lobby and choose", "%Nif you want to be a Hider or a Seeker!", "&61 time use.",},
            "shop.blockHuntPassV2Description"),

    SIGN_LEAVE(new String[]{"%H[" + BlockHunt.getPluginDescriptionFile().getName() + "%H]", "&4LEAVE", "&8Right-Click", "&8To leave."},
            "sign.leave"),
    SIGN_SHOP(new String[]{"%H[" + BlockHunt.getPluginDescriptionFile().getName() + "%H]", "&4SHOP", "&8Right-Click", "&8To shop."},
            "sign.shop"),
    SIGN_WAITING(new String[]{"%H[" + BlockHunt.getPluginDescriptionFile().getName() + "%H]", "%A%arenaName%",
            "%A%players%%N/%A%maxplayers%", "&8Waiting..."}, "sign.waiting"),
    SIGN_STARTING(new String[]{"%H[" + BlockHunt.getPluginDescriptionFile().getName() + "%H]", "%A%arenaName%",
            "%A%players%%N/%A%maxplayers%", "&2Start: %A%timeleft%"}, "sign.starting"),
    SIGN_IN_GAME(new String[]{"%H[" + BlockHunt.getPluginDescriptionFile().getName() + "%H]", "%A%arenaName%",
            "%A%players%%N/%A%maxplayers%", "%EInGame: %A%timeleft%"}, "sign.inGame"),

    SCOREBOARD_ENABLED(true, "scoreboard.enabled"),
    SCOREBOARD_TITLE("%H[" + BlockHunt.getPluginDescriptionFile().getName() + "]", "scoreboard.title"),
    SCOREBOARD_TIME_LEFT("%ATime left:", "scoreboard.timeLeft"),
    SCOREBOARD_SEEKERS("%NSeekers:", "scoreboard.seekers"),
    SCOREBOARD_HIDERS("%NHiders:", "scoreboard.hiders"),

    REQUIRE_INVENTORY_CLEAR_ON_JOIN(false, "requireInventoryClearOnJoin"),
    ;

    private final Object defaultValue;
    private final ConfigManager configManager = MemoryStorage.config;
    private final String path;

    /**
     * Instantiates a new config key
     *
     * @param defaultValue <p>The default value of this config key</p>
     * @param path         <p>The path of this value in the configuration file</p>
     */
    ConfigKey(Object defaultValue, String path) {
        this.defaultValue = defaultValue;
        this.path = path;
    }

    @Override
    public String getPath() {
        return this.path;
    }

    @Override
    public ConfigManager getConfigManager() {
        return this.configManager;
    }

    @Override
    public Object getDefaultValue() {
        return defaultValue;
    }

}
