package net.knarcraft.blockhunt.listener;

import net.knarcraft.blockhunt.InventoryHandler;
import net.knarcraft.blockhunt.MemoryStorage;
import net.knarcraft.blockhunt.arena.Arena;
import net.knarcraft.blockhunt.arena.ArenaHandler;
import net.knarcraft.blockhunt.arena.ArenaProperty;
import net.knarcraft.blockhunt.config.ConfigKey;
import net.knarcraft.blockhunt.config.MessageKey;
import net.knarcraft.blockhunt.manager.MessageManager;
import org.apache.commons.lang.WordUtils;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.event.inventory.InventoryType;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.InventoryView;
import org.bukkit.inventory.ItemStack;

public class OnInventoryClickEvent implements Listener {

    @EventHandler(priority = EventPriority.NORMAL, ignoreCancelled = true)
    public void onInventoryClickEvent(InventoryClickEvent event) {
        Player player = (Player) event.getWhoClicked();

        for (Arena arena : MemoryStorage.arenaList) {
            if (arena.playersInArena.contains(player)) {
                event.setCancelled(true);
            }
        }

        Inventory inv = event.getInventory();
        InventoryView invView = event.getView();
        if (inv.getType().equals(InventoryType.CHEST)) {
            if (invView.getTitle().contains("DisguiseBlocks")) {
                if (event.getCurrentItem() != null) {
                    if (!event.getCurrentItem().getType().isBlock()) {
                        if (!event.getCurrentItem().getType().equals(Material.FLOWER_POT)) {
                            event.setCancelled(true);
                            MessageManager.sendFMessage(player, MessageKey.ERROR_SET_NOT_A_BLOCK);
                        }
                    }
                }

                return;
            }

            // Early exit if this isn't a block-hunt inventory
            if (!invView.getTitle().contains("BlockHunt")) {
                return;
            }

            if (invView.getTitle().startsWith("\u00A7r")) {
                if (invView.getTitle().equals(MessageManager.replaceAll("\u00A7r" + MemoryStorage.config.get(ConfigKey.SHOP_TITLE)))) {
                    event.setCancelled(true);
                    ItemStack item = event.getCurrentItem();
                    if (MemoryStorage.shop.getFile().get(player.getName() + ".tokens") == null) {
                        MemoryStorage.shop.getFile().set(player.getName() + ".tokens", 0);
                        MemoryStorage.shop.save();
                    }
                    int playerTokens = MemoryStorage.shop.getFile().getInt(player.getName() + ".tokens");
                    if (item == null) {
                        return;
                    }
                    if (item.getType().equals(Material.AIR)) {
                        return;
                    }
                    if (item.getItemMeta().getDisplayName() == null) {
                        return;
                    }
                    if (item.getItemMeta().getDisplayName().equals(MessageManager.replaceAll(MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_CHOOSER_V_1_NAME).toString()))) {
                        if (playerTokens >= (Integer) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_CHOOSER_V_1_PRICE)) {
                            MemoryStorage.shop.getFile().set(player.getName() + ".blockchooser", true);
                            MemoryStorage.shop.getFile().set(player.getName() + ".tokens", playerTokens - (Integer) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_CHOOSER_V_1_PRICE));
                            MemoryStorage.shop.save();
                            MessageManager.sendFMessage(player, MessageKey.NORMAL_SHOP_BOUGHT_ITEM, "itemName-" + MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_CHOOSER_V_1_NAME));
                        } else {
                            MessageManager.sendFMessage(player, MessageKey.ERROR_SHOP_NEED_MORE_TOKENS);
                        }
                    } else if (item.getItemMeta().getDisplayName().equals(MessageManager.replaceAll(MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_HUNT_PASS_V_2_NAME).toString()))) {
                        if (playerTokens >= (Integer) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_HUNT_PASS_V_2_PRICE)) {
                            if (MemoryStorage.shop.getFile().get(player.getName() + ".blockhuntpass") == null) {
                                MemoryStorage.shop.getFile().set(player.getName() + ".blockhuntpass", 0);
                                MemoryStorage.shop.save();
                            }

                            MemoryStorage.shop.getFile().set(player.getName() + ".blockhuntpass", (Integer) MemoryStorage.shop.getFile().get(player.getName() + ".blockhuntpass") + 1);
                            MemoryStorage.shop.getFile().set(player.getName() + ".tokens", playerTokens - (Integer) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_HUNT_PASS_V_2_PRICE));
                            MemoryStorage.shop.save();
                            MessageManager.sendFMessage(player, MessageKey.NORMAL_SHOP_BOUGHT_ITEM, "itemName-" + MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_HUNT_PASS_V_2_NAME));
                        } else {
                            MessageManager.sendFMessage(player, MessageKey.ERROR_SHOP_NEED_MORE_TOKENS);
                        }
                    }

                    InventoryHandler.openShop(player);
                } else if (invView.getTitle().contains(MessageManager.replaceAll((String) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_CHOOSER_V_1_NAME)))) {
                    event.setCancelled(true);
                    if (event.getCurrentItem() != null) {
                        if (event.getCurrentItem().getType().isBlock()) {
                            MemoryStorage.chosenBlock.put(player, event.getCurrentItem());
                            String blockName = event.getCurrentItem().getType().name();
                            blockName = WordUtils.capitalizeFully(blockName.replace("_", " "));
                            MessageManager.sendFMessage(player, MessageKey.NORMAL_SHOP_CHOSEN_BLOCK, "block-"
                                    + blockName);
                        } else {
                            MessageManager.sendFMessage(player, MessageKey.ERROR_SET_NOT_A_BLOCK);
                        }
                    }
                } else if (invView.getTitle().contains(MessageManager.replaceAll((String) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_HUNT_PASS_V_2_NAME)))) {
                    event.setCancelled(true);
                    if (event.getCurrentItem() != null) {
                        if (event.getCurrentItem().getType().equals(Material.BLUE_WOOL)) {
                            int i = 0;
                            for (Arena arena : MemoryStorage.arenaList) {
                                if (arena.playersInArena.contains(player)) {
                                    for (Player playerCheck : arena.playersInArena) {
                                        if (MemoryStorage.chosenSeeker.get(playerCheck) != null) {
                                            if (MemoryStorage.chosenSeeker.get(playerCheck)) {
                                                i = i + 1;
                                            }
                                        }
                                    }
                                }

                                if (i >= arena.amountSeekersOnStart) {
                                    MessageManager.sendFMessage(player, MessageKey.ERROR_SHOP_MAX_SEEKERS_REACHED);
                                } else {
                                    MemoryStorage.chosenSeeker.put(player, true);
                                    player.getInventory().setItemInMainHand(new ItemStack(Material.AIR));
                                    player.updateInventory();
                                    MessageManager.sendFMessage(player, MessageKey.NORMAL_SHOP_CHOSEN_SEEKER);
                                    inv.clear();
                                    if (MemoryStorage.shop.getFile().getInt(player.getName() + ".blockhuntpass") == 1) {
                                        MemoryStorage.shop.getFile().set(player.getName() + ".blockhuntpass", null);
                                        MemoryStorage.shop.save();
                                    } else {
                                        MemoryStorage.shop.getFile().set(player.getName() + ".blockhuntpass", MemoryStorage.shop.getFile().getInt(player.getName() + ".blockhuntpass") - 1);
                                        MemoryStorage.shop.save();
                                    }
                                }
                            }

                        } else if (event.getCurrentItem().getType().equals(Material.RED_WOOL)) {
                            int i = 0;
                            for (Arena arena : MemoryStorage.arenaList) {
                                if (arena.playersInArena.contains(player)) {
                                    for (Player playerCheck : arena.playersInArena) {
                                        if (MemoryStorage.chosenSeeker.get(playerCheck) != null) {
                                            if (!MemoryStorage.chosenSeeker.get(playerCheck)) {
                                                i = i + 1;
                                            }
                                        }
                                    }
                                }

                                if (i >= (arena.playersInArena.size() - 1)) {
                                    MessageManager.sendFMessage(player, MessageKey.ERROR_SHOP_MAX_HIDERS_REACHED);
                                } else {
                                    MemoryStorage.chosenSeeker.put(player, false);
                                    player.getInventory().setItemInMainHand(new ItemStack(Material.AIR));
                                    player.updateInventory();
                                    MessageManager.sendFMessage(player, MessageKey.NORMAL_SHOP_CHOSEN_HIDERS);
                                    inv.clear();
                                    if (MemoryStorage.shop.getFile().getInt(player.getName() + ".blockhuntpass") == 1) {
                                        MemoryStorage.shop.getFile().set(player.getName() + ".blockhuntpass", null);
                                        MemoryStorage.shop.save();
                                    } else {
                                        MemoryStorage.shop.getFile().set(player.getName() + ".blockhuntpass", MemoryStorage.shop.getFile().getInt(player.getName() + ".blockhuntpass") - 1);
                                        MemoryStorage.shop.save();
                                    }
                                }
                            }
                        }
                    }
                } else {
                    event.setCancelled(true);
                    ItemStack item = event.getCurrentItem();
                    String arenaName = inv.getItem(0).getItemMeta().getDisplayName().replaceAll(MessageManager.replaceAll("%NBlockHunt arena: %A"), "");

                    Arena arena = null;
                    for (Arena arena2 : MemoryStorage.arenaList) {
                        if (arena2.arenaName.equalsIgnoreCase(arenaName)) {
                            arena = arena2;
                        }
                    }

                    if (item == null) {
                        return;
                    }
                    if (item.getType().equals(Material.AIR)) {
                        return;
                    }
                    if (!item.getItemMeta().hasDisplayName()) {
                        return;
                    }
                    if (item.getType().equals(Material.GOLD_NUGGET)) {
                        if (item.getItemMeta().getDisplayName().contains("maxPlayers")) {
                            updownButton(player, item, arena, ArenaProperty.MAX_PLAYERS, arena.maxPlayers, Bukkit.getMaxPlayers(), 2, 1, 1);
                        } else if (item.getItemMeta().getDisplayName().contains("minPlayers")) {
                            updownButton(player, item, arena, ArenaProperty.MIN_PLAYERS, arena.minPlayers, Bukkit.getMaxPlayers() - 1, 2, 1, 1);
                        } else if (item.getItemMeta().getDisplayName().contains("amountSeekersOnStart")) {
                            updownButton(player, item, arena, ArenaProperty.AMOUNT_SEEKERS_ON_START, arena.amountSeekersOnStart, arena.maxPlayers - 1, 1, 1, 1);
                        } else if (item.getItemMeta().getDisplayName().contains("timeInLobbyUntilStart")) {
                            updownButton(player, item, arena, ArenaProperty.TIME_IN_LOBBY_UNTIL_START, arena.timeInLobbyUntilStart, 1000, 5, 1, 1);
                        } else if (item.getItemMeta().getDisplayName().contains("waitingTimeSeeker")) {
                            updownButton(player, item, arena, ArenaProperty.WAITING_TIME_SEEKER, arena.waitingTimeSeeker, 1000, 5, 1, 1);
                        } else if (item.getItemMeta().getDisplayName().contains("gameTime")) {
                            updownButton(player, item, arena, ArenaProperty.GAME_TIME, arena.gameTime, 1000, 5, 1, 1);
                        } else if (item.getItemMeta().getDisplayName().contains("blockAnnouncerTime")) {
                            updownButton(player, item, arena, ArenaProperty.BLOCK_ANNOUNCER_TIME, arena.blockAnnouncerTime, 1000, 0, 5, 5);
                        } else if (item.getItemMeta().getDisplayName().contains("timeUntilHidersSword")) {
                            updownButton(player, item, arena, ArenaProperty.TIME_UNTIL_HIDERS_SWORD, arena.timeUntilHidersSword, 1000, 0, 1, 1);
                        } else if (item.getItemMeta().getDisplayName().contains("hidersTokenWin")) {
                            updownButton(player, item, arena, ArenaProperty.HIDERS_TOKEN_WIN, arena.hidersTokenWin, 1000, 0, 1, 1);
                        } else if (item.getItemMeta().getDisplayName().contains("seekersTokenWin")) {
                            updownButton(player, item, arena, ArenaProperty.SEEKERS_TOKEN_WIN, arena.seekersTokenWin, 1000, 0, 1, 1);
                        } else if (item.getItemMeta().getDisplayName().contains("killTokens")) {
                            updownButton(player, item, arena, ArenaProperty.KILL_TOKENS, arena.killTokens, 1000, 0, 1, 1);
                        }

                        save(arena);
                        InventoryHandler.openPanel(player, arena.arenaName);

                    } else if (item.getType().equals(Material.BOOK)) {
                        if (item.getItemMeta().getDisplayName().contains("disguiseBlocks")) {
                            InventoryHandler.openDisguiseBlocks(arena, player);
                        }
                    }
                }
            }
        }
    }

    public void save(Arena arena) {
        MemoryStorage.arenas.getFile().set(arena.arenaName, arena);
        MemoryStorage.arenas.save();
        ArenaHandler.loadArenas();
    }

    public static void updownButton(Player player, ItemStack item, Arena arena, ArenaProperty at, int option, int max, int min, int add, int remove) {
        if (item.getItemMeta().getDisplayName().contains((String) MemoryStorage.messages.get(MessageKey.BUTTON_ADD_2))) {
            if (option < max) {
                switch (at) {
                    case MAX_PLAYERS -> arena.maxPlayers = option + add;
                    case MIN_PLAYERS -> arena.minPlayers = option + add;
                    case AMOUNT_SEEKERS_ON_START -> arena.amountSeekersOnStart = option + add;
                    case TIME_IN_LOBBY_UNTIL_START -> arena.timeInLobbyUntilStart = option + add;
                    case WAITING_TIME_SEEKER -> arena.waitingTimeSeeker = option + add;
                    case GAME_TIME -> arena.gameTime = option + add;
                    case BLOCK_ANNOUNCER_TIME -> arena.blockAnnouncerTime = option + add;
                    case TIME_UNTIL_HIDERS_SWORD -> arena.timeUntilHidersSword = option + add;
                    case HIDERS_TOKEN_WIN -> arena.hidersTokenWin = option + add;
                    case SEEKERS_TOKEN_WIN -> arena.seekersTokenWin = option + add;
                    case KILL_TOKENS -> arena.killTokens = option + add;
                }
            } else {
                MessageManager.sendFMessage(player, MessageKey.ERROR_SET_TOO_HIGH_NUMBER, "max-" + max);
            }
        } else if (item.getItemMeta().getDisplayName().contains((String) MemoryStorage.messages.get(MessageKey.BUTTON_REMOVE_2))) {
            if (option > min) {
                switch (at) {
                    case MAX_PLAYERS -> arena.maxPlayers = option - remove;
                    case MIN_PLAYERS -> arena.minPlayers = option - remove;
                    case AMOUNT_SEEKERS_ON_START -> arena.amountSeekersOnStart = option - remove;
                    case TIME_IN_LOBBY_UNTIL_START -> arena.timeInLobbyUntilStart = option - remove;
                    case WAITING_TIME_SEEKER -> arena.waitingTimeSeeker = option - remove;
                    case GAME_TIME -> arena.gameTime = option - remove;
                    case BLOCK_ANNOUNCER_TIME -> arena.blockAnnouncerTime = option - remove;
                    case TIME_UNTIL_HIDERS_SWORD -> arena.timeUntilHidersSword = option - remove;
                    case HIDERS_TOKEN_WIN -> arena.hidersTokenWin = option - remove;
                    case SEEKERS_TOKEN_WIN -> arena.seekersTokenWin = option - remove;
                    case KILL_TOKENS -> arena.killTokens = option - remove;
                }
            } else {
                MessageManager.sendFMessage(player, MessageKey.ERROR_SET_TOO_LOW_NUMBER, "min-" + min);
            }
        }
    }
}
