package net.knarcraft.blockhunt.manager;

import net.knarcraft.blockhunt.BlockHunt;
import net.knarcraft.blockhunt.config.MessageKey;
import net.knarcraft.blockhunt.config.Permission;
import net.knarcraft.blockhunt.config.PermissionDefault;
import org.bukkit.entity.Player;

public class PermissionsManager {

    /**
     * Check if a player has the permission. Also checks * related permissions.
     * This permission DOES HAVE TO be registered.
     *
     * @param player  The player who needs the permission.
     * @param perm    The permission.
     * @param message Send a message to the player saying you don't have the
     *                permission.
     * @return True/False when either the player has the permission.
     */
    public static boolean hasPerm(Player player, Permission perm, Boolean message) {
        PermissionDefault type = perm.type;
        if (player == null) {
            return true;
        }

        if (type == PermissionDefault.ALL) {
            return true;
        } else if (type == PermissionDefault.OP) {
            if (player.isOp()) {
                return true;
            }
        } else if (type == PermissionDefault.ADMIN) {
            if (player.hasPermission(BlockHunt.plugin.rootPermission + "admin")) {
                return true;
            }
        } else if (type == PermissionDefault.MODERATOR) {
            if (player.hasPermission(BlockHunt.plugin.rootPermission + "moderator")) {
                return true;
            }
        } else if (type == PermissionDefault.PLAYER) {
            if (player.hasPermission(BlockHunt.plugin.rootPermission + "player")) {
                return true;
            }
        }

        if (player.hasPermission("*")) {
            return true;
        } else if (player.hasPermission(BlockHunt.plugin.rootPermission + "*")) {
            return true;
        } else if (player.hasPermission(BlockHunt.plugin.rootPermission + perm.perm)) {
            return true;
        } else if (player.hasPermission(BlockHunt.plugin.rootPermission + perm.perm + ".*")) {
            return true;
        } else {
            if (message) {
                MessageManager.sendFMessage(player, MessageKey.ERROR_NO_PERMISSION);
            }
        }
        return false;
    }

    /**
     * Check if a player has the permission. Also checks * related permissions.
     * This permission does NOT have to be registered.
     *
     * @param player  The player who needs the permission.
     * @param type    Type permission.
     * @param perm    The permission.
     * @param message Send a message to the player saying you don't have the
     *                permission.
     * @return True/False when either the player has the permission.
     */
    public static boolean hasRawPerm(Player player, PermissionDefault type, String perm, Boolean message) {
        if (player == null) {
            return true;
        }

        if (type == PermissionDefault.ALL) {
            return true;
        } else if (type == PermissionDefault.OP) {
            if (player.isOp()) {
                return true;
            }
        } else if (type == PermissionDefault.ADMIN) {
            if (player.hasPermission(BlockHunt.plugin.rootPermission + "admin")) {
                return true;
            }
        } else if (type == PermissionDefault.MODERATOR) {
            if (player.hasPermission(BlockHunt.plugin.rootPermission + "moderator")) {
                return true;
            }
        } else if (type == PermissionDefault.PLAYER) {
            if (player.hasPermission(BlockHunt.plugin.rootPermission + "player")) {
                return true;
            }
        }

        if (player.hasPermission("*")) {
            return true;
        } else if (player.hasPermission(BlockHunt.plugin.rootPermission + "*")) {
            return true;
        } else if (player.hasPermission(perm)) {
            return true;
        } else if (player.hasPermission(perm)) {
            return true;
        } else {
            if (message) {
                MessageManager.sendFMessage(player, MessageKey.ERROR_NO_PERMISSION);
            }
        }
        return false;
    }
}
